@@ -2,7 +2,6 @@
 """
 CLI Plugin for handling OBJ
 """
-import argparse
 import getpass
 import glob
 import math
@@ -11,11 +10,19 @@ import platform
 import socket
 import sys
 import time
+from argparse import ArgumentParser, ArgumentTypeError
+from contextlib import suppress
 from datetime import datetime
 from math import ceil
+from typing import List
 
 from terminaltables import SingleTable
 
+from linodecli.cli import CLI
+from linodecli.configuration import _do_get_request
+from linodecli.configuration.helpers import _default_thing_input
+from linodecli.plugins import PluginContext
+
 ENV_ACCESS_KEY_NAME = "LINODE_CLI_OBJ_ACCESS_KEY"
 ENV_SECRET_KEY_NAME = "LINODE_CLI_OBJ_SECRET_KEY"
 
@@ -82,9 +89,9 @@ def restricted_int_arg_type(max, min=1):  # pylint: disable=redefined-builtin
         except ValueError as e:
             # argparse can handle ValueErrors, but shows an unfriendly "invalid restricted_int
             # value: '0.1'" message, so catch and raise with a better message.
-            raise argparse.ArgumentTypeError(err_msg) from e
+            raise ArgumentTypeError(err_msg) from e
         if value < min or value > max:
-            raise argparse.ArgumentTypeError(err_msg)
+            raise ArgumentTypeError(err_msg)
         return value
 
     return restricted_int
@@ -94,7 +101,7 @@ def list_objects_or_buckets(get_client, args):
     """
     Lists buckets or objects
     """
-    parser = argparse.ArgumentParser(PLUGIN_BASE + " ls")
+    parser = ArgumentParser(PLUGIN_BASE + " ls")
 
     parser.add_argument(
         "bucket",
@@ -132,7 +139,11 @@ def list_objects_or_buckets(get_client, args):
                 size = c.size
 
             # pylint: disable-next=redefined-outer-name
-            datetime = _convert_datetime(c.last_modified) if size != "DIR" else " " * 16
+            datetime = (
+                _convert_datetime(c.last_modified)
+                if size != "DIR"
+                else " " * 16
+            )
 
             data.append([datetime, size, c.name])
 
@@ -157,10 +168,13 @@ def create_bucket(get_client, args):
     """
     Creates a new bucket
     """
-    parser = argparse.ArgumentParser(PLUGIN_BASE + " mb")
+    parser = ArgumentParser(PLUGIN_BASE + " mb")
 
     parser.add_argument(
-        "name", metavar="NAME", type=str, help="The name of the bucket to create."
+        "name",
+        metavar="NAME",
+        type=str,
+        help="The name of the bucket to create.",
     )
 
     parsed = parser.parse_args(args)
@@ -176,10 +190,13 @@ def delete_bucket(get_client, args):
     """
     Deletes a bucket
     """
-    parser = argparse.ArgumentParser(PLUGIN_BASE + " rb")
+    parser = ArgumentParser(PLUGIN_BASE + " rb")
 
     parser.add_argument(
-        "name", metavar="NAME", type=str, help="The name of the bucket to remove."
+        "name",
+        metavar="NAME",
+        type=str,
+        help="The name of the bucket to remove.",
     )
     parser.add_argument(
         "--recursive",
@@ -214,18 +231,22 @@ def upload_object(get_client, args):  # pylint: disable=too-many-locals
     """
     Uploads an object to object storage
     """
-    parser = argparse.ArgumentParser(PLUGIN_BASE + " put")
+    parser = ArgumentParser(PLUGIN_BASE + " put")
 
     parser.add_argument(
         "file", metavar="FILE", type=str, nargs="+", help="The files to upload."
     )
     parser.add_argument(
-        "bucket", metavar="BUCKET", type=str, help="The bucket to put a file in."
+        "bucket",
+        metavar="BUCKET",
+        type=str,
+        help="The bucket to put a file in.",
     )
     parser.add_argument(
         "--acl-public",
         action="store_true",
-        help="If set, the new object can be downloaded without " "authentication.",
+        help="If set, the new object can be downloaded without "
+        "authentication.",
     )
     parser.add_argument(
         "--chunk-size",
@@ -288,10 +309,14 @@ def upload_object(get_client, args):  # pylint: disable=too-many-locals
         k.key = filename
 
         print(filename)
-        k.set_contents_from_filename(file_path, cb=_progress, num_cb=100, policy=policy)
+        k.set_contents_from_filename(
+            file_path, cb=_progress, num_cb=100, policy=policy
+        )
 
     for filename, file_path, file_size in to_multipart_upload:
-        _do_multipart_upload(bucket, filename, file_path, file_size, policy, chunk_size)
+        _do_multipart_upload(
+            bucket, filename, file_path, file_size, policy, chunk_size
+        )
 
     print("Done.")
 
@@ -345,8 +370,7 @@ def _do_multipart_upload(
                             time.sleep(retry_delay)
                             continue
                         raise
-                    else:
-                        break
+                    break
     except Exception:
         print("Upload failed!  Cleaning up!")
         upload.cancel_upload()
@@ -359,7 +383,7 @@ def get_object(get_client, args):
     """
     Retrieves an uploaded object and writes it to a file
     """
-    parser = argparse.ArgumentParser(PLUGIN_BASE + " get")
+    parser = ArgumentParser(PLUGIN_BASE + " get")
 
     parser.add_argument(
         "bucket", metavar="BUCKET", type=str, help="The bucket the file is in."
@@ -407,7 +431,7 @@ def delete_object(get_client, args):
     """
     Removes a file from a bucket
     """
-    parser = argparse.ArgumentParser(PLUGIN_BASE + " del")
+    parser = ArgumentParser(PLUGIN_BASE + " del")
 
     parser.add_argument(
         "bucket", metavar="BUCKET", type=str, help="The bucket to delete from."
@@ -442,10 +466,13 @@ def generate_url(get_client, args):
     """
     Generates a URL to an object
     """
-    parser = argparse.ArgumentParser(PLUGIN_BASE + " signurl")
+    parser = ArgumentParser(PLUGIN_BASE + " signurl")
 
     parser.add_argument(
-        "bucket", metavar="BUCKET", type=str, help="The bucket containing the object."
+        "bucket",
+        metavar="BUCKET",
+        type=str,
+        help="The bucket containing the object.",
     )
     parser.add_argument(
         "file", metavar="OBJECT", type=str, help="The object to sign a URL to."
@@ -493,7 +520,7 @@ def set_acl(get_client, args):
     """
     Modify Access Control List for a Bucket or Objects
     """
-    parser = argparse.ArgumentParser(PLUGIN_BASE + " setacl")
+    parser = ArgumentParser(PLUGIN_BASE + " setacl")
 
     parser.add_argument(
         "bucket", metavar="BUCKET", type=str, help="The bucket to modify."
@@ -512,7 +539,9 @@ def set_acl(get_client, args):
         help="If given, makes the target publicly readable.",
     )
     parser.add_argument(
-        "--acl-private", action="store_true", help="If given, makes the target private."
+        "--acl-private",
+        action="store_true",
+        help="If given, makes the target private.",
     )
 
     parsed = parser.parse_args(args)
@@ -553,7 +582,7 @@ def enable_static_site(get_client, args):
     """
     Turns a bucket into a static website
     """
-    parser = argparse.ArgumentParser(PLUGIN_BASE + " ws-create")
+    parser = ArgumentParser(PLUGIN_BASE + " ws-create")
 
     parser.add_argument(
         "bucket",
@@ -597,7 +626,7 @@ def static_site_info(get_client, args):
     """
     Returns info about a configured static site
     """
-    parser = argparse.ArgumentParser(PLUGIN_BASE + " ws-info")
+    parser = ArgumentParser(PLUGIN_BASE + " ws-info")
 
     parser.add_argument(
         "bucket",
@@ -632,7 +661,7 @@ def show_usage(get_client, args):
     """
     Shows space used by all buckets in this cluster, and total space
     """
-    parser = argparse.ArgumentParser(PLUGIN_BASE + " du")
+    parser = ArgumentParser(PLUGIN_BASE + " du")
 
     parser.add_argument(
         "bucket",
@@ -668,7 +697,9 @@ def show_usage(get_client, args):
 
         total = _denominate(total)
 
-        tab = _borderless_table([[_pad_to(total, length=7), f"{num} objects", b.name]])
+        tab = _borderless_table(
+            [[_pad_to(total, length=7), f"{num} objects", b.name]]
+        )
         print(tab.table)
 
     if len(buckets) > 1:
@@ -694,10 +725,14 @@ def _denominate(total):
     return total
 
 
-def list_all_objects(get_client):
+def list_all_objects(get_client, args):
     """
     Lists all objects in all buckets
     """
+    # this is for printing help when --help is in the args
+    parser = ArgumentParser(PLUGIN_BASE + " la")
+    parser.parse_args(args)
+
     client = get_client()
 
     # all buckets
@@ -725,7 +760,7 @@ def disable_static_site(get_client, args):
     """
     Disables static site for a bucket
     """
-    parser = argparse.ArgumentParser(PLUGIN_BASE + " du")
+    parser = ArgumentParser(PLUGIN_BASE + " du")
 
     parser.add_argument(
         "bucket",
@@ -770,22 +805,39 @@ COMMAND_MAP = {
 }
 
 
-def call(args, context):  # pylint: disable=too-many-branches,too-many-statements
+def print_help(parser: ArgumentParser):
     """
-    This is called when the plugin is invoked
+    Print out the help info to the standard output.
     """
-    if not HAS_BOTO:
-        # we can't do anything - ask for an install
-        pip_version = "pip3" if sys.version[0] == 3 else "pip"
+    parser.print_help()
 
-        print(
-            "This plugin requires the 'boto' module.  Please install it by running "
-            f"'{pip_version} install boto'"
-        )
+    # additional help
+    print()
+    print("Available commands: ")
 
-        sys.exit(2)  # requirements not met - we can't go on
+    command_help_map = [
+        [name, func.__doc__.strip()]
+        for name, func in sorted(COMMAND_MAP.items())
+    ]
 
-    parser = argparse.ArgumentParser(PLUGIN_BASE, add_help=False)
+    tab = SingleTable(command_help_map)
+    tab.inner_heading_row_border = False
+    print(tab.table)
+    print()
+    print(
+        "Additionally, you can regenerate your Object Storage keys using the "
+        "'regenerate-keys' command or configure defaults for the plugin using "
+        "the 'configure' command."
+    )
+    print()
+    print("See --help for individual commands for more information")
+
+
+def get_obj_args_parser():
+    """
+    Initialize and return the argument parser for the obj plug-in.
+    """
+    parser = ArgumentParser(PLUGIN_BASE, add_help=False)
     parser.add_argument(
         "command",
         metavar="COMMAND",
@@ -799,50 +851,44 @@ def call(args, context):  # pylint: disable=too-many-branches,too-many-statement
         type=str,
         help="The cluster to use for the operation",
     )
+    return parser
 
+
+def call(
+    args: List[str], context: PluginContext
+):  # pylint: disable=too-many-branches,too-many-statements
+    """
+    This is called when the plugin is invoked
+    """
+    if not HAS_BOTO:
+        # we can't do anything - ask for an install
+        print(
+            "This plugin requires the 'boto' module.  Please install it by running "
+            "'pip3 install boto' or 'pip install boto'"
+        )
+
+        sys.exit(2)  # requirements not met - we can't go on
+
+    parser = get_obj_args_parser()
     parsed, args = parser.parse_known_args(args)
 
     # don't mind --no-defaults if it's there; the top-level parser already took care of it
-    try:
+    with suppress(ValueError):
         args.remove("--no-defaults")
-    except ValueError:
-        pass
 
     if not parsed.command:
-        # show help if invoked with no command
-        parser.print_help()
-
-        # additional help
-        print()
-        print("Available commands: ")
-
-        command_help_map = [
-            [name, func.__doc__.strip()] for name, func in sorted(COMMAND_MAP.items())
-        ]
-
-        tab = SingleTable(command_help_map)
-        tab.inner_heading_row_border = False
-        print(tab.table)
-        print()
-        print(
-            "Additionally, you can regenerate your Object Storage keys using the "
-            "'regenerate-keys' command or configure defaults for the plugin using "
-            "the 'configure' command."
-        )
-        print()
-        print("See --help for individual commands for more information")
-
+        print_help(parser)
         sys.exit(0)
 
     # make a client, but only if we weren't printing help
 
     access_key, secret_key = (
-        os.environ.get(ENV_ACCESS_KEY_NAME, None),
-        os.environ.get(ENV_SECRET_KEY_NAME, None),
+        os.getenv(ENV_ACCESS_KEY_NAME, None),
+        os.getenv(ENV_SECRET_KEY_NAME, None),
     )
 
     if not "--help" in args:
-        if access_key and not secret_key or secret_key and not access_key:
+        if bool(access_key) != bool(secret_key):
             print(
                 f"You must set both {ENV_ACCESS_KEY_NAME} and {ENV_SECRET_KEY_NAME}, or neither"
             )
@@ -933,7 +979,7 @@ def _get_boto_client(cluster, access_key, secret_key):
     return client
 
 
-def _get_s3_creds(client, force=False):
+def _get_s3_creds(client: CLI, force: bool = False):
     """
     Retrieves stored s3 creds for the acting user from the config, or generates new
     creds using the client and stores them if none exist
@@ -1022,23 +1068,25 @@ def _get_s3_creds(client, force=False):
 
         client.config.plugin_set_value("access-key", access_key)
         client.config.plugin_set_value("secret-key", secret_key)
-        client.config.write_config(silent=True)
+        client.config.write_config()
 
     return access_key, secret_key
 
 
-def _configure_plugin(client):
+def _configure_plugin(client: CLI):
     """
     Configures a default cluster value.
     """
     clusters = [
         c["id"]
-        for c in client.config._do_get_request(  # pylint: disable=protected-access
-            "/object-storage/clusters", token=client.config.get_value("token")
+        for c in _do_get_request(  # pylint: disable=protected-access
+            client.config.base_url,
+            "/object-storage/clusters",
+            token=client.config.get_value("token"),
         )["data"]
     ]
 
-    cluster = client.config._default_thing_input(  # pylint: disable=protected-access
+    cluster = _default_thing_input(  # pylint: disable=protected-access
         "Configure a default Cluster for operations.",
         clusters,
         "Default Cluster: ",
@@ -1086,10 +1134,14 @@ def _convert_datetime(datetime_str):
     """
     Given a string in INCOMING_DATE_FORMAT, returns a string in DATE_FORMAT
     """
-    return datetime.strptime(datetime_str, INCOMING_DATE_FORMAT).strftime(DATE_FORMAT)
+    return datetime.strptime(datetime_str, INCOMING_DATE_FORMAT).strftime(
+        DATE_FORMAT
+    )
 
 
-def _pad_to(val, length=10, right_align=False):  # pylint: disable=unused-argument
+def _pad_to(
+    val, length=10, right_align=False
+):  # pylint: disable=unused-argument
     """
     Pads val to be at minimum length characters long
     """
